==> crashops_flutter-0.0.826.tar.gz: android/src/main/kotlin/com/crashops/crashops_flutter/CrashOpsPlugin.kt <==
package com.crashops.crashops_flutter

import androidx.annotation.NonNull
import android.os.Bundle

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

import com.crashops.sdk.CrashOps

/** CrashOpsPlugin */
public class CrashOpsPlugin: FlutterPlugin, MethodCallHandler {
  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val channel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "crashops_flutter")
    channel.setMethodCallHandler(CrashOpsPlugin())
  }

  companion object {
    private lateinit var previousLogs: MutableList<String>

    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "crashops_flutter")
      channel.setMethodCallHandler(CrashOpsPlugin())

      CrashOps.getInstance().setPreviousLogsListener { previousLogs ->
        this.previousLogs = previousLogs
      }
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "CrashOps#isEnabled" -> {
        result.success(CrashOps.getInstance().isEnabled)
      }

      "CrashOps#setEnabled" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is Boolean) {
            val isEnabled = args
            if (isEnabled) {
              CrashOps.getInstance().enable()
            } else {
              CrashOps.getInstance().disable()
            }

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setEnabled` must get a boolean as an argument."
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      "CrashOps#setMetadata" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is HashMap<*, *>) {
            val metadata: Bundle = args.toBundleOfStrings()
            CrashOps.getInstance().setMetadata(metadata)

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setEnabled` must get a boolean as an argument."
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      "CrashOps#setClientId" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is String) {
            val clientId = args
            CrashOps.getInstance().setClientId(clientId)

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setClientId` must get a String as an argument."
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      "CrashOps#onFlutterError" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is HashMap<*, *>) {
            val errorDetailsBundle: Bundle = args.toBundleOfStrings()
            CrashOps.getInstance().logError("Error from Flutter's app", errorDetailsBundle)

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setEnabled` must get a boolean as an argument"
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
  }
}

private fun <K, V> java.util.HashMap<K, V>.toBundleOfStrings(): Bundle {
  val bundle = Bundle()
  forEach {
    if (it.value is ArrayList<*>) {
      (it.value as? ArrayList<*>)?.let { arr ->
        val strings: ArrayList<String> = arrayListOf()
        arr.forEach { element ->
          // TODO: What should we do in case of nested bundles / a map entry that has an array?
          when (element) {
//            is Bundle -> {
//              strings.add((element as? Bundle)?.toMap()?.toBundleOfStrings())
//            }
//            is HashMap<*, *> -> {
//              strings.add(element.toBundleOfStrings())
//            }
            else -> strings.add(element.toString())
          }
        }

        bundle.putStringArrayList(it.key.toString(), strings)
      }
    } else if (it.value is Bundle) {
      bundle.putBundle(it.key.toString(), (it.value as? Bundle)?.toMap()?.toBundleOfStrings())
    } else {
      bundle.putString(it.key.toString(), it.value.toString())
    }
  }

  return bundle
}


private fun Bundle.toMap(): HashMap<String, Any> {
  val map: java.util.HashMap<String, Any> = hashMapOf()

  this.keySet().forEach {
    this.get(it)?.let { value ->
      if (value is Bundle) {
        map.put(it, value.toMap())
      } else {
        map.put(it, value)
      }
    }
  }

  return map
}

==> crashops_flutter-0.2.1.tar.gz: android/src/main/kotlin/com/crashops/crashops_flutter/CrashOpsPlugin.kt <==
package com.crashops.crashops_flutter

import androidx.annotation.NonNull
import android.os.Bundle

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

import com.crashops.sdk.CrashOps

/** CrashOpsPlugin */
public class CrashOpsPlugin: FlutterPlugin, MethodCallHandler {
  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val channel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "crashops_flutter")
    channel.setMethodCallHandler(CrashOpsPlugin())
  }

  companion object {
    private lateinit var previousLogs: MutableList<String>

    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "crashops_flutter")
      channel.setMethodCallHandler(CrashOpsPlugin())

      CrashOps.getInstance().setPreviousLogsListener { previousLogs ->
        this.previousLogs = previousLogs
      }
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "CrashOps#isEnabled" -> {
        result.success(CrashOps.getInstance().isEnabled)
      }

      "CrashOps#setEnabled" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is Boolean) {
            val isEnabled = args
            if (isEnabled) {
              CrashOps.getInstance().enable()
            } else {
              CrashOps.getInstance().disable()
            }

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setEnabled` must get a boolean as an argument."
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      "CrashOps#setMetadata" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is HashMap<*, *>) {
            val metadata: Bundle = args.toBundleOfStrings()
            CrashOps.getInstance().setMetadata(metadata)

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setEnabled` must get a boolean as an argument."
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      "CrashOps#setAppKey" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is String) {
            val appKey = args
            CrashOps.getInstance().setAppKey(appKey)

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setAppKey` must get a String as an argument."
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      "CrashOps#onFlutterError" -> {
        var didHandle = false
        call.arguments?.let { args ->
          if (args is HashMap<*, *>) {
            val errorDetailsBundle: Bundle = args.toBundleOfStrings()
            CrashOps.getInstance().logError("Error from Flutter's app", errorDetailsBundle)

            didHandle = true
          }
        }

        if (didHandle) {
          result.success("OK")
        } else {
          val message = "`setEnabled` must get a boolean as an argument"
          val code = "crashops-error-code"
          result.error(code, message, "details (Any?)")
        }
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
  }
}

private fun <K, V> java.util.HashMap<K, V>.toBundleOfStrings(): Bundle {
  val bundle = Bundle()
  forEach {
    if (it.value is ArrayList<*>) {
      (it.value as? ArrayList<*>)?.let { arr ->
        val strings: ArrayList<String> = arrayListOf()
        arr.forEach { element ->
          // TODO: What should we do in case of nested bundles / a map entry that has an array?
          when (element) {
//            is Bundle -> {
//              strings.add((element as? Bundle)?.toMap()?.toBundleOfStrings())
//            }
//            is HashMap<*, *> -> {
//              strings.add(element.toBundleOfStrings())
//            }
            else -> strings.add(element.toString())
          }
        }

        bundle.putStringArrayList(it.key.toString(), strings)
      }
    } else if (it.value is Bundle) {
      bundle.putBundle(it.key.toString(), (it.value as? Bundle)?.toMap()?.toBundleOfStrings())
    } else {
      bundle.putString(it.key.toString(), it.value.toString())
    }
  }

  return bundle
}


private fun Bundle.toMap(): HashMap<String, Any> {
  val map: java.util.HashMap<String, Any> = hashMapOf()

  this.keySet().forEach {
    this.get(it)?.let { value ->
      if (value is Bundle) {
        map.put(it, value.toMap())
      } else {
        map.put(it, value)
      }
    }
  }

  return map
}
